#ifndef IPBLOCK_INC
#define IPBLOCK_INC

#include "ipCore.h"

namespace Torch {

/** This class is designed to decompose an image into blocks and apply an ipCore to each block

    \begin{verbatim}
                                +--------------------------+
                                |                          |                       
                                |         ipBlock          |
                                |                          |
                                |                          |
                                |   +--+      +--------+   |
				|   |AA|  --> | ipCore | --|--> seq_out->frames[0]
				|   |AA|      +--------+   |
	                        |   +--+                   |
			        |			   |	  
       seq_in->frames[0]        |   +--+      +--------+   |
   	           +----+       |   |BB|  --> | ipCore | --|--> seq_out->frames[1]
		   |AABB|       |   |BB|      +--------+   |
	           |AABB|  ---> |   +--+                   |
                   |CCDD|       |                          |
                   |CCDD|       |   +--+      +--------+   |
                   +----+       |   |CC|  --> | ipCore | --|--> seq_out->frames[2]
                                |   |CC|      +--------+   |
                                |   +--+                   |
                                |                          |
                                |   +--+      +--------+   |
                                |   |DD|  --> | ipCore | --|--> seq_out->frames[3]
                                |   |DD|      +--------+   |
                                |   +--+                   |
                                |                          |
                                +--------------------------+
    \end{verbatim}
    
    @author Sebastien Marcel (marcel@idiap.ch)
    @author Yann Rodriguez   (rodrig@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/	
class ipBlock : public ipCore
{
   	// input image size
	int width, height;

	// subimage
	Sequence *subimage;

	// computes internal variables
	void init_(void);
public:
	/// difference between #block_size# and #block_overlap#
	int pixeladv;
	
	/// row offset
	int row_offset;
	
	/// column offset
	int col_offset;

	/// number of rows (lines)
	int rowblocks;

	/// number of columns
	int colblocks;

	/// number of blocks
	int n_block;
	
	/// block size (square)
	int block_size;

	/// overlap between block [0...block-1]
	int block_overlap;

	/// ipCore used to process each block
	ipCore *ip;
	
	//-----
	
	/** constructor
	
	    @param width_ is the #width# of the input image
	    @param height_ is the #height# of the input image
	    @param coding_ is the #coding# format of the input image
	    @param ip_ is the ipCore to apply to every block
	    @param ip_output_size_ is the output size of the ipCore #ip_#
	    @param block_size_ is the size of blocks
	    @param block_overlap_ is the overlap between blocks
	*/
	ipBlock(int width_, int height_, const char *coding_, ipCore *ip_, int ip_output_size_, int block_size_, int block_overlap_);

	///
	virtual ~ipBlock();
	
	///
        virtual void process(Sequence *seq_in);
};

}

#endif
