#ifndef IPCORE_INC
#define IPCORE_INC

#include "vision.h"

namespace Torch {

/** This class is designed to handle image processing algorithms

    An ipCore process an input (1 image, several images, 1 feature vector or several feature vectors)
    and outputs (1 image, several images, 1 feature vector or several feature vectors).

    In a ipCore, there is necessary an image in input or in output.
    
    \begin{verbatim}

                                +--------------+
		                |              |
	       seq_in     ----> |    ipCore    | ----> seq_out
        (images or features)    |              |  (images or features)
                                +--------------+
    
    \end{verbatim}
    
    @author Yann Rodriguez (rodrig@idiap.ch)
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class ipCore : public Object
{
protected:
	bool FindImageFormat(const char *coding_);

public:	
	/**@name output field */
	//@{
	/** output the ipCore.
	
	    If the output represents images, each frames of #seq_out#
	    corresponds to one image. If the output represents features,
	    each frame corresponds to a feature vector.
	    
	    WARNING: each ipCore is responsible to allocate #seq_out#
	*/
	Sequence *seq_out;
	//@}
	
	/**@name input/output image coding field */
	//@{
	/// image coding: gray(0), rgb(1), yuv(2), float(3)
	char *coding;

	/// image coding id: gray(0), rgb(1), yuv(2), float(3)
	int coding_id;

	/// number of planes
	int n_planes;
	
	//@}
	
	/**@name other fields */
	//@{
	bool verbose;
	//@}

	//-----
	
	/**@name constructor and destructor */
	//@{
	/** creates an ipCore to process images.
	    
	    @param coding_ is the coding of the image to process ("gray", "rgb", "yuv", "float")
	*/
	ipCore(const char *coding_);
	
	/// destructor
	virtual ~ipCore();
	//@}
	
	/**@name initialization and processing */
	//@{
	/// initialize the ipCore
	virtual void init(void);

	/// process the input #seq_in# and produces #seq_out#
        virtual void process(Sequence *seq_in);
	//@}
	
	/**@name accessing and modifying output */
	//@{
	/// get the output width
	virtual int getWidthOut(void);
	
	/// get the output height
	virtual int getHeightOut(void);
	
	/// set the output size
	virtual void setOutputSize(int width_, int height_);

	/** returns the output frame number #frame_#

	    @param frame_ is the number of the frame to access
	    @return the output frame #frame_# (seq_out->frames[#frame_#])
	*/
	real *getOutput(int frame_) { return seq_out->frames[frame_]; };
	//@}
};

}

#endif
