#include "ipFFT2D.h"

namespace Torch {

double **allocmatrix__(int h, int w, Allocator *allocator)
{
	double **m;

	m = (double **)allocator->alloc(h*sizeof(double*));
	m -= 1;
	
	for(int i=1 ; i <= h; i++)
	{
		m[i] = (double *)allocator->alloc(w*sizeof(double));
		m[i] -= 1;
	}
	   
	return m;
}

ipFFT2D::ipFFT2D(int width_, int height_, bool inverse_) : ipCore("gray")
{
   	width = width_;
	height = height_;
	inverse = inverse_;

	if((width < 8) || (height < 8))
		error("ipFFT2D(): Image size should not be smaller than 8.");

	if(width != (int) pow(2.0, ceil(log2((double)width))))
		error("ipFFT2D(): width is not a power of 2.");
	   
	if(height != (int) pow(2.0, ceil(log2((double)height))))
		error("ipFFT2D(): height is not a power of 2.");
	   
	N = width * height;

	seq_out = new(allocator) Sequence(2, N);

	tmp1 = (double *)allocator->alloc(2*width*sizeof(double));
	tmp1 -= 1;
	tmp2 = (double *)allocator->alloc(2*height*sizeof(double));
	tmp2 -= 1;
	T = allocmatrix__(2*height, width, allocator);
	R = allocmatrix__(height, width, allocator);
	I = allocmatrix__(height, width, allocator);
	Fr = allocmatrix__(height, width, allocator);
	Fi = allocmatrix__(height, width, allocator);
}

void ipFFT2D::process(Sequence *seq_in)
{
	real *dst_r = seq_out->frames[0];
	real *dst_i = seq_out->frames[1];

	if(seq_in->n_frames == 0 || seq_in->n_frames > 2) error("ipFFT2D::process() incorrect input number of frames (%d)\n", seq_in->n_frames);
	
	for(int i=1 ; i <= height ; i++) 
	{
		int offset = (i-1)*width;
		
		for(int j=1 ; j <= width ; j++) 
		{
                	R[i][j] = seq_in->frames[0][offset+j-1];

			if(seq_in->n_frames == 2) I[i][j] = seq_in->frames[1][offset+j-1];
			else I[i][j] = 0.0;
		}
	}

	// 2D FFT
	if(inverse) four2(Fr, Fi, R, I, height, width,-1);
	else four2(Fr, Fi, R, I, height, width, 1);
	
	for(int i=1 ; i <= height ; i++) 
	{
		int offset = (i-1)*width;
		
		for(int j=1 ; j <= width ; j++) 
		{
		   	if(inverse)
			{
                		dst_r[offset+j-1] = Fr[i][j]/(double)N;
                		dst_i[offset+j-1] = Fi[i][j]/(double)N;
			}
			else
			{
                		dst_r[offset+j-1] = Fr[i][j];
                		dst_i[offset+j-1] = Fi[i][j];
			}
		}
	}
}

#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr

void ipFFT2D::four1(double *data, int nn, int isign)
{
	int n, mmax, m, j, istep, i;
	double wtemp, wr, wpr, wpi, wi, theta;
	double tempr, tempi;

	n = nn << 1;
	j = 1;
	for (i=1;i<n;i+=2) {
		if (j > i) {
			SWAP(data[j],data[i]);
			SWAP(data[j+1],data[i+1]);
		}
		m = n >> 1;
		while (m >= 2 && j > m) {
			j -= m;
			m >>= 1;
		}
		j += m;
	}
	mmax = 2;
	while (n > mmax) {
		istep = 2*mmax;
		theta = 6.28318530717959/(isign*mmax);
		wtemp = sin(0.5*theta);
		wpr = -2.0*wtemp*wtemp;
		wpi = sin(theta);
		wr = 1.0;
		wi = 0.0;
		for (m=1;m<mmax;m+=2) {
			for (i=m;i<=n;i+=istep) {
				j = i+mmax;
				tempr = wr*data[j]-wi*data[j+1];
				tempi = wr*data[j+1]+wi*data[j];
				data[j] = data[i]-tempr;
				data[j+1] = data[i+1]-tempi;
				data[i] += tempr;
				data[i+1] += tempi;
			}
			wr = (wtemp=wr)*wpr-wi*wpi+wr;
			wi = wi*wpr+wtemp*wpi+wi;
		}
		mmax = istep;
	}
}

void ipFFT2D::four2(double **fftr, double **ffti, double **rdata, double **idata, int row, int col, int isign)
{
        int i, j;

        for (i=1;i<=row;i++) 
	{
            for (j=1;j<=col;j++) 
	    {
                tmp1[j*2-1] = rdata[i][j];
                tmp1[j*2] = idata[i][j];
            }
	    
            four1(tmp1, col, isign);
	    
            for (j=1;j<=col;j++) 
	    {
                T[i*2-1][j] = tmp1[j*2-1];
                T[i*2][j] = tmp1[j*2];
            }
        }

        for (i=1;i<=col;i++) 
	{
            for (j=1;j<=row;j++) 
	    {
                tmp2[j*2-1] = T[j*2-1][i];
                tmp2[j*2] = T[j*2][i];
            }
	    
            four1(tmp2,row,isign);
	    
            for (j=1;j<=row;j++) 
	    {
                fftr[j][i] = tmp2[j*2-1];
                ffti[j][i] = tmp2[j*2];
            }
        }
}

ipFFT2D::~ipFFT2D()
{
}

}

