#ifndef IPFFT2D_INC
#define IPFFT2D_INC

#include "ipCore.h"

namespace Torch {

/** This class is designed to compute the 2D Fast Fourier Transform

    Thanks to Souheil Ben Yacoub for the original source code.

    \begin{verbatim}

      seq_in->frames[0]        +--------------+
	       +----+          |              |
	       |    |   ---->  |   ipFFT2D    | ----> seq_out->frames[0] (FFT coefficients real part)
	       |    |          |              |       seq_out->frames[1] (FFT coefficients imag part)
	       +----+          +--------------+
          (power of 2 image
        or FFT2D coefficients)
	 
    \end{verbatim}
    
    @author Sebastien Marcel (marcel@idiap.ch)
    
    @version 2.0
    \Date
    @since 2.0
*/
class ipFFT2D : public ipCore
{
   	// image size
   	int N;
	
	double **R, **I, **Fr, **Fi;
        double **T, *tmp1, *tmp2;
	
protected:

	/// 1D Fourier transform
   	void four1(double *data, int nn, int isign);

	/** 2D Fourier transform of an image
	
	    @param rdata is the real part of the image
	    @param idata is the imaginary part of the image
	    @param row is the #height# of the image
	    @param col is the #width# of the image
	    @param fftr is the real part of the result
	    @param ffti is the imaginary part of the result
	    @param isign compute FFT is isign=1, compute inverse FFT if isign=-1
	*/
	void four2(double **fftr, double **ffti, double **rdata, double **idata, int row, int col, int isign);
	
public:
	/// width of data
	int width;

	/// height of data
	int height;

	/// perform FFT inverse
	bool inverse;
	
	//----
	
	///
	ipFFT2D(int width_, int height_, bool inverse_ = false);

	///
    	virtual ~ipFFT2D();

	///
    	virtual void process(Sequence *seq_in);

};

}

#endif
