#include "tiffDiskXFile.h"

namespace Torch {

   
tiffDiskXFile::tiffDiskXFile(const char *file_name, const char *open_flags)
{
   	n_planes = 3;
	coding = "rgb";

#ifdef HAVE_TIFF
	if(!strcmp(open_flags, "r"))
	{
		tif = TIFFOpen(file_name, "r");
		raster = NULL;
		scanline_buf = NULL;
	}
	else if(!strcmp(open_flags, "w"))
	{
		tif = TIFFOpen(file_name, "w");
		raster = NULL;
		scanline_buf = NULL;
		bpp = 3;
	}
	else error("tiffDiskXFile::tiffDiskXFile(%s, %s) incorrect open flag.", file_name, open_flags);
#else
	error("tiffDiskXFile::tiffDiskXFile() TIFF format not supported.");
#endif
}

bool tiffDiskXFile::readHeader(int *width_, int *height_)
{
#ifdef HAVE_TIFF
	TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &width);
    	TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &height);
	TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, &bpp);

    	npixels = width * height;

	if(raster != NULL) _TIFFfree(raster);
    	raster = (uint32 *)_TIFFmalloc(npixels * sizeof(uint32));
    	if(!raster) return false;

	if(verbose)
    		print("reading TIFF file: %dx%d (%d-bit) pixels\n", width, height, 8*bpp);

	if(pixmap != NULL) delete [] pixmap;
	pixmap = new unsigned char [3 * width * height];
	
	*width_ = width;
	*height_ = height;

	return true;
#else
	return false;
#endif
}

int tiffDiskXFile::readPixmap()
{
#ifdef HAVE_TIFF
    	TIFFReadRGBAImage(tif, width, height, raster, true);
    
    	unpack_tiff_raster(raster);

	return width * height * 3;
#else
	return -1;
#endif
}

bool tiffDiskXFile::writeHeader(int width_, int height_)
{
   	width = width_;
	height = height_;
  
#ifdef HAVE_TIFF
	scanline_size = bpp * width;

	TIFFSetField(tif, TIFFTAG_IMAGEWIDTH, width);
	TIFFSetField(tif, TIFFTAG_IMAGELENGTH, height);
	TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, bpp);
	TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 8);
    	TIFFSetField(tif, TIFFTAG_ROWSPERSTRIP, width * bpp);
	TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
	TIFFSetField(tif, TIFFTAG_COMPRESSION, COMPRESSION_NONE);
	TIFFSetField(tif, TIFFTAG_FILLORDER, FILLORDER_MSB2LSB);
	TIFFSetField(tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
    
	if(TIFFScanlineSize(tif) != scanline_size)
	{
		warning("tiffDiskXFile::writeHeader() Mismatch with library's expected scanline size.\n");
		return false;
    	}
    
	if(scanline_buf != NULL) free(scanline_buf);
    	scanline_buf = (char *)malloc(scanline_size);
    	if(!scanline_buf) 
	{
		warning("tiffDiskXFile::writeHeader() Can't allocate scanline buffer.\n");
		return false;
    	}
	
	return true;
#else
	return false;
#endif
}

int tiffDiskXFile::writePixmap(unsigned char *pixmap_)
{
#ifdef HAVE_TIFF
   	char *scanline;

	scanline = (char *) pixmap_;
    	for(int y = 0 ; y < height ; y++)
    	{
		memcpy(scanline_buf, scanline, scanline_size);
		
		TIFFWriteScanline(tif, scanline_buf, y, 0);

		scanline += scanline_size;
    	}
    
	return width * height * 3;
#else
	return -1;
#endif
}

tiffDiskXFile::~tiffDiskXFile()
{
#ifdef HAVE_TIFF
	TIFFClose(tif);
    	if(raster != NULL) _TIFFfree(raster);
	if(scanline_buf != NULL) free(scanline_buf);
#endif
	if(pixmap != NULL) delete [] pixmap;
}

#ifdef HAVE_TIFF
void tiffDiskXFile::unpack_tiff_raster(uint32 *raster)
{
	int x,y;
	uint32 pixel;
	uint32 pic_index = 0;

	for(y=(height-1); y>=0; y--)
		for(x=0; x<width; x++)
		{
	    		pixel = raster[y*width + x];

	    		if(bpp == 1)
	    		{
	       			unsigned char r_ = TIFFGetR(pixel);
		
	    			pixmap[pic_index++] = r_;
	    			pixmap[pic_index++] = r_;
	    			pixmap[pic_index++] = r_;
	    		}
	    		else if(bpp == 3)
	    		{
	    			pixmap[pic_index++] = TIFFGetR(pixel);
	    			pixmap[pic_index++] = TIFFGetG(pixel);
	    			pixmap[pic_index++] = TIFFGetB(pixel);
	    		}
		}
}
#endif

}
