#include "ipRelaxation.h"
#include "multigrid.h"
#include "ImageGray.h"
#include "matrix_addons.h"

namespace Torch {


ipRelaxation::ipRelaxation(real lambda_, int type_, int steps_, int width_, int height_, const char *coding_) : ipCore(coding_)
{
  width = width_;
  height = height_;
  lambda = lambda_;
  type = type_;
  steps = steps_;

  rho = (real*) allocator->alloc (5*sizeof(real));

  image = new(allocator) Vec(width*height);
  light = new(allocator) Vec(width*height);

  image->zero();
  light->zero();

  seq_out = new(allocator) Sequence(2, width*height);

  for(int i=0; i<seq_out->n_frames; i++)
     for(int j=0; j<seq_out->frame_size; j++)
       seq_out->frames[i][j] = 0.0; 
}



void ipRelaxation::process(Sequence *seq_in)
{
  switch(coding_id)
    {
    case 0:
      {	
	
	// initialization
	// "source" term: image
	// solution: light
	for (int i=0; i<width*height; i++){
	  image->ptr[i] = seq_in->frames[0][i];
	  light->ptr[i] = seq_in->frames[0][i];
	}

	// apply relaxation steps (gaussSeidel -> see multigrid.cc)
	for (int i=0; i<=steps; i++) {
	  gaussSeidel(light, image, rho, lambda, type, width, height);

	  // swap: the improved estimate becomes the new estimate for next relaxation step
	  image = light;
	}
	
	// rescale the vector 
	vecScale(light, 255);
	
	// build final result (R = I/L)
	for (int i=0; i<width*height; i++) {
	
	  if ((i < width) || (i >= ((height - 1)*width)) ||  ((i % width) < 1) || ((i % width) >= (width-1))) {
	    seq_out->frames[0][i] = 1;
	  }
	    
	  else {
	      
	    if (IS_NEAR(light->ptr[i], 0.0, 1)) {
	      seq_out->frames[0][i] = 1;
	    }
	    else{
		seq_out->frames[0][i] = ((seq_in->frames[0][i])/(light->ptr[i]));
	    }
	  }
	  seq_out->frames[1][i] = light->ptr[i];
	}
	  
	// display purpose
	cutExtremum(seq_out->frames[0], width, height, 4);
	
	// rescale between 0-255
	scaleGray(width, height, seq_out->frames[0]);
	scaleGray(width, height, seq_out->frames[1]);

	break;
      }
      
	
    case 1:
      message("ipAnisoDirect::process() not implemented for RGB format");
      break;
    case 2:
      message("ipAnisoDirect::process() not implemented for YUV format");
      break;
    case 3:
      message("ipAnisoDirect::process() not implemented for float format");
      break;
    }
}

void ipRelaxation::cutExtremum(real *data, int width, int height, int distribution_width) {
  
  // used to 'cut' the extreme of the pixels distribution in the result
  real mean_out = 0.0;
  real var_out = 0.0;
  real std_dev = 0.0; 
  
  // compute the mean
  for (int i=0; i<width*height; i++)
    mean_out += data[i];   
  mean_out /= width*height;
  
  
  // compute variance and standard deviation
  for (int i=0; i<width*height; i++)
    var_out += (data[i]- mean_out)*(data[i]- mean_out);    
  var_out /= (width*height - 1);
  std_dev = sqrt(var_out);
  
  
  int count = 0;
  
  for (int i=0; i<width*height; i++) {
    
    if (data[i] > mean_out + distribution_width*std_dev) {
      data[i] = mean_out + distribution_width*std_dev;
      count++;
      //print("up saturation : %g\n", mean_out + distribution_width*std_dev);		
    }
    
    if (data[i] < mean_out - distribution_width*std_dev){
      data[i] = mean_out - distribution_width*std_dev;
      count++;
      //print("down saturation: %g\n",  mean_out - distribution_width*std_dev);
    }
  }
}


ipRelaxation::~ipRelaxation() 
{
}

}
