#include "Serial.h"

namespace Torch {

Serial::Serial(char *device_, char port_)
{
	device = (char *) malloc(sizeof(char) * (strlen(device_) + 2));
	
	sprintf(device, "%s%c", device_, port_);

	bVerbose = false;
}

Serial::~Serial()
{
   	free(device);
}

bool Serial::connect()
{
	struct termios termdata;

	if(bVerbose == true)
   		printf("Serial::open device=%s\n", device);
	
	//
   	handle = open(device, O_RDWR | O_NDELAY);
	
   	if(handle < 0)
   	{
		fprintf(stderr, "Serial::open Error: impossible to open device %s\n", device);

		return false;
   	}
     
	if(bVerbose == true)
		printf("handle = %d\n", handle);
	
	//
  	if(isatty(handle) == 0) 
	{
    		fprintf(stderr, "Serial::open Error: %s is not a terminal\n", device);

		return false;
  	}
  
	//
  	if(tcgetattr(handle, &termdata) < 0) 
	{
    		fprintf(stderr, "Serial::open Error: tcgetattr\n");

		return false;
  	}

        tcflush(handle, TCIFLUSH);
        
//        termdata.c_cflag = B9600 | CS8 |CREAD | CSTOPB | CLOCAL;
        termdata.c_cflag = CS8 |CREAD | CSTOPB | CLOCAL;
        
        cfsetispeed(&termdata, B9600);
        cfsetospeed(&termdata, B9600);
        tcsetattr(handle, TCSANOW, &termdata);
 
	//
  	if(cfsetispeed(&termdata, B9600) < 0) 
	{
    		fprintf(stderr, "Serial::open Error: tcsetispeed\n");

		return false;
  	}

	//
  	if(cfsetospeed(&termdata, B9600) < 0) 
	{
    		fprintf(stderr, "Serial::open Error: tcsetospeed\n");

		return false;
  	}

	//
  	if(tcsetattr(handle, TCSANOW, &termdata) < 0 ) 
	{
    		fprintf(stderr, "Serial::open Error: tcsetattr\n");

		return false;
	}
	
	// Does specific device initialization
	return init();
}

void Serial::printCmd(sCommandPacket *command)
{
   	printf("\n|");
  	for(int i = 0 ; i < command->length ; i++) printf("%2x|", command->packet[i]);
  	printf("(length=%d)\n", command->length);
}

int Serial::send(sCommandPacket *command) 
{
  	int written = 0;
	int thisTime = 0;
	int length = command->length;
  	unsigned char *pos;

	if(bVerbose == true)
	{
		printf("handle = %d\n", handle);
		printCmd(command);
	}

  	while((written < length) && (thisTime != -1)) 
	{
    		pos = command->packet + sizeof(char)*written;  // The current position

		if(bVerbose == true)
			for(int i = 0 ; i < length - written ; i++) printf(" -> |%2x|\n", pos[i]);
		
    		thisTime = write(handle, pos, length - written); // Write as many bytes as pos.
    		written += thisTime; // Update the number of bytes written
  	}

	if(bVerbose == true)
		printf("Serial::send done.\n");

  	return (thisTime < 0 ? -1 : written); 
}

bool Serial::sendCmd(sCommandPacket *command, bool ack, bool cpl)
{
     	sCommandPacket retCommand;
  	int retrys = 0;
	int retVal;
  	int number_of_times;
  
   	if(ack == true) send(command);
	else
	{
  		// Send command and wait for an answer, retransmit if necessary
  		do 
		{
    			number_of_times = 0;

    			// Send a command
    			while(send(command) != command->length) 
			{
      				if(number_of_times < RETRIES) 
				{
        				number_of_times++;
        				printf("Serial::sendCmd: Retransmitting\n");
      				}
      				else 
				{
        				fprintf(stderr, "Serial::sendCmd Giving up on sending data\n");

        				return false; // Failed to send the command
      				}
    			} // End of while

    			// Read from the camera. An acknowledgement is expected
    			if(waitingForReply(2, &retCommand) < 0) 
			{
      				fprintf(stderr, "Serial::sendCmd: Giving up on reading data.\n");
      				printCmd(&retCommand);
				
      				return false;  
    			}
			
    			retrys++;
  		} while((retVal=IsAck(&retCommand)) == 0 && retrys < RETRIES);
  
  		// Error detection
		if(!retVal)
		{
    			fprintf(stderr, "Serial::sendCmd: answer not acknowledge-signal.\n");
    			printCmd(&retCommand);
			
    			return false;
  		}
	} // End of if (ack == true)

  
	if(cpl == true)
	{
  		// Wait for an complete signal. Retransmit if necessary
  		do 
		{
    			// Read from the camera. A complete signal is expected
    			if(waitingForReply(5, &retCommand) < 0) 
			{
      				fprintf(stderr, "Serial::sendCmd: Giving up on reading data..\n");
      				printCmd(&retCommand);
				
      				return false;  
    			}
     
			retrys++;
  		} while((retVal=IsCpl(&retCommand)) == 0 && retrys < RETRIES);

  		// Error detection

  		if(!retVal) 
  		{
    			fprintf(stderr, "%s", "SendCmd: Answer not an complete-signal.\n");
  	  		printCmd(&retCommand);
			
    			return false;
  		}
	}
	
	return true;
}

int Serial::readCmd(sCommandPacket *command)
{
  	int readTillNow = 0;
	int thisTime = 1;
	int number_of_times = 0;

	if(bVerbose == true)
	{
		printf("Serial::readCmd ...\n");
		printf("handle = %d\n", handle);
	}

    	// Read the sequence from a handle
  	do 
  	{
    		thisTime = read(handle, &(command->packet[readTillNow]), 1); // Read one byte

		if(thisTime == -1) 
		{
			if(bVerbose == true)
			{
				perror("Serial::readCmd");
				printf("%s\n", strerror(errno));
			}
		}
		
		if(bVerbose == true)
			printf(" -> |%2x| [%d] (%d)\n", command->packet[readTillNow], readTillNow, thisTime);
		
    		if(thisTime > 0) readTillNow++; // The number of bytes read
    		number_of_times++;
    		if(number_of_times == 100) return -1;
  	} while(thisTime > -1 && readTillNow < MAXPACKETLENGTH && ((command->packet[readTillNow-1] != byteTerminator()) || (readTillNow == 0)));

  	command->length = readTillNow; // The length of the sequence

  	return (thisTime < 0 ? -1 : readTillNow); 
}

int Serial::waitingForReply(int timeInSec, sCommandPacket *command)
{
  	int retVal;
	int thisTime = 0;
  	fd_set readFds;
  	struct timeval timeout;
 
	if(bVerbose == true)
		printf("Serial::waitingForReply %ds\n", timeInSec);
	
  	FD_ZERO(&readFds);
  	FD_SET(handle, &readFds);
	
  	timeout.tv_sec = timeInSec; // Setup time to wait
  	timeout.tv_usec = 0;
	
  	retVal = select(handle+1, &readFds, NULL, NULL, &timeout);
  	if(retVal > 0) 
	{
    		while((retVal=readCmd(command)) < 1) 
		{
      			thisTime++;
      			if(thisTime == 5) return -1;
    		}
    
		if(bVerbose == true)
			printCmd(command);

		return retVal;   
  	}

  	return -1; // Error code
}

int Serial::isEqual(sCommandPacket *command1, sCommandPacket *command2) 
{
  	int i;
  	int retVal = 1;
  
  	if(command1->length == command2->length) 
	{ 
		// The same length ?
    		for(i = 0 ; i < command1->length && retVal ; i++) 
       			retVal = retVal && (command2->packet[i] == command1->packet[i]); // The same content ? 
    		return retVal;
  	}
  	else return 0;  
}

}

