#include "IOBufferize.h"
#include "FileBinDataSet.h"
#include "IOMulti.h"
#include "IOBin.h"
#include "IOSub.h"

namespace Torch {

FileBinDataSet::FileBinDataSet(char *filename, int n_inputs_, real target, int max_load, bool verbose_)
{
   	if(verbose_)
	{
		message("FileBinDataSet::FileBinDataSet()");
		print("\t+ one file = %s\n", filename);
		print("\t+ n_inputs expected = %d\n", n_inputs_);
		print("\t+ target to assign = %g\n", target);
	}

  	if(n_inputs_ <= 0)
    		error("FileBinDataSet: n_inputs must be positive");

  	IOSequence *io_inputs = NULL;

	//
  	// Firstly the inputs ...
  	//----------------------------
   	if(verbose_)
  		message("FileBinDataSet: allocating inputs ...");
  	IOBin *io_file = NULL;
  	io_file = new(allocator) IOBin(filename, false, max_load, false);

  	io_inputs = io_file;  // the file doesn't contain targets.. (by definition of a bindata file)

  	if(io_inputs->frame_size != n_inputs_)
    		error("FileBinDataSet: n_inputs provided doesn't correspond to the n_inputs of the bindata file");


	//
  	// ... and then the targets (here it's the same real target for all patterns of the file)
  	//-------------------------------------------------------------------------------------------
  	Sequence **the_targets = (Sequence **)allocator->alloc(sizeof(Sequence *)*io_file->n_sequences);
  	for(int j = 0; j < io_file->n_sequences; j++)
  	{
    		the_targets[j] = new(allocator) Sequence(1, 1);  // Matrix 1x1 ... for the target (real)
    		the_targets[j]->frames[0][0] = target;
  	}

	//
  	// now load the inputs using the init() method and load the targets with setTargets()
  	//------------------------------------------------------------------------------------------- 
  	MemoryDataSet::init(io_inputs, NULL);  

	// targets added with setTargets() method
  	setTargets(the_targets, io_file->n_sequences);

   	if(verbose_)
  		message("FileBinDataSet: %d examples loaded assigned with target %g", n_examples, target);
}
	
FileBinDataSet::FileBinDataSet(char *filename,  int n_inputs_, int max_load, bool verbose_)
{
   	if(verbose_)
	{
		message("FileBinDataSet::FileBinDataSet()");
		print("\t+ one file = %s\n", filename);
		print("\t+ n_inputs expected = %d\n", n_inputs_);
	}

  	if(n_inputs_ <= 0)
    		error("FileBinDataSet: n_inputs must be positive");
    
  	IOSequence *io_inputs = NULL;

	//
  	// Here we have only the inputs ...
  	//----------------------------
   	if(verbose_)
  		message("FileBinDataSet: allocating inputs ...");
  	IOBin *io_file = NULL;
  	io_file = new(allocator) IOBin(filename, false, max_load, false);

  	io_inputs = io_file;  // the file doesn't contain targets.. (by definition of a bindata file)

  	if(io_inputs->frame_size != n_inputs_)
    		error("FileBinDataSet: n_inputs provided doesn't correspond to the n_inputs of the bindata file");
    

  	// now load the inputs using the init() method 
  	//--------------------------------------------- 
  	MemoryDataSet::init(io_inputs, NULL); 

   	if(verbose_)
  		message("FileBinDataSet: %d examples loaded with no targets assign", n_examples);
}

FileBinDataSet::FileBinDataSet(char **filenames, int n_files, int n_inputs_, real *targets, int max_load, bool verbose_)
{
	message("FileBinDataSet::FileBinDataSet()");
	print("\t+ %d files\n", n_files);
	print("\t+ n_inputs expected = %d\n", n_inputs_);
	print("\t+ %d targets assigned\n", n_files);

  	if(n_files <= 0)
    		error("FileBinDataSet: check the number of files");

  	if(n_inputs_ <= 0)
    		error("FileBinDataSet: n_inputs must be positive");

  	IOSequence *io_inputs = NULL;
  	IOSequence *io_targets = NULL;

	
	//
  	// Firstly the inputs ...
  	//----------------------------
   	if(verbose_)
  		message("FileBinDataSet: allocating inputs ...");
  	IOBin **io_files = (IOBin **)allocator->alloc(sizeof(IOBin *)*n_files);
  	for(int i = 0; i < n_files; i++)
  	{
    		io_files[i] = new(allocator) IOBin(filenames[i], false, max_load, false);
    	
   		if(verbose_)
		{
			print("file [%d] = %s\n", i, filenames[i]);
			print("\t+ n_examples = %d\n", io_files[i]->n_sequences);
			print("\t+ n_inputs   = %d\n", io_files[i]->frame_size);
		}
  	}
  
  	IOMulti *io_file = new(allocator) IOMulti((IOSequence **)io_files, n_files);

  	io_inputs = io_file;  // the "concatenated" file doesn't contain targets.. (by definition of a bindata file)

	//
  	// ... and then the targets (same real target for all patterns of the same file[i])
  	//-------------------------------------------------------------------------------------------
   	if(verbose_)
  		message("FileBinDataSet: allocating targets ...");
  	Sequence **the_targets = (Sequence **)allocator->alloc(sizeof(Sequence *)*io_file->n_sequences);
  	int index = 0;
  	for(int i = 0; i < n_files; i++)
  	{
    		for(int j = 0; j < io_files[i]->n_sequences; j++)
    		{
      			the_targets[index] = new(allocator) Sequence(1, 1);  // Matrix 1x1 ... for the target (real)
      			the_targets[index]->frames[0][0] = targets[i];
      			index++;
    		}
  	}
   

  	// now load the inputs using the init() method and load the targets with setTargets()
 	//------------------------------------------------------------------------------------------- 
 	if(verbose_)
  		message("FileBinDataSet: loading inputs ...");
  	MemoryDataSet::init(io_inputs, io_targets);

 	if(verbose_)
  		message("FileBinDataSet: setting targets ...");
  	setTargets(the_targets, io_file->n_sequences);

 	if(verbose_)
  		message("FileBinDataSet: %d examples loaded from %d files with %d targets assigned", n_examples, n_files, n_files);
}


FileBinDataSet::FileBinDataSet(char **filenames, int n_files, int n_inputs_, int offset_window, int n_inputs_window, real *targets, int max_load, bool verbose_)
{
	message("FileBinDataSet::FileBinDataSet() (loading a subset of consecutive columns)");
	print("\t+ %d files\n", n_files);
	print("\t+ n_inputs expected = %d\n", n_inputs_);
	print("\t+ offset_window = %d\n", offset_window);
	print("\t+ size_window = %d\n", n_inputs_window);
	print("\t+ %d targets assigned\n", n_files);

  	if(n_files <= 0)
    		error("FileBinDataSet: check the number of files!");

  	if(n_inputs_ <= 0)
    		error("FileBinDataSet: n_inputs must be positive!");

  	IOSequence *io_inputs = NULL;
  	IOSequence *io_targets = NULL;

	//
  	// Firstly the inputs ...
  	//----------------------------
 	if(verbose_)
  		message("FileBinDataSet: allocating inputs ...");
  	IOSequence **io_files = (IOSequence **)allocator->alloc(sizeof(IOSequence *)*n_files);
  	for(int i = 0; i < n_files; i++)
  	{
    		//io_files[i] = new(allocator) IOBin(filenames[i], false, max_load, false);
   		
	   	IOBin *io_bin = new(allocator) IOBin(filenames[i], false, max_load, false);
		IOBufferize *io_buffer = new(allocator) IOBufferize(io_bin);
		io_files[i] = new(allocator) IOSub(io_buffer, offset_window, n_inputs_window);
		
 		if(verbose_)
		{
			print("file [%d] = %s\n", i, filenames[i]);
			print("\t+ n_examples      = %d\n", io_files[i]->n_sequences);
			print("\t+ n_inputs        = %d\n", io_files[i]->frame_size);
			print("\t+ n_inputs_window = %d\n", n_inputs_window);
		}
  	}
  
  	IOMulti *io_file = new(allocator) IOMulti(io_files, n_files);

  	io_inputs = io_file;  // the "concatenated" file doesn't contain targets.. (by definition of a bindata file)


	//
  	// ... and then the targets (same real target for all patterns of the same file[i])
  	//-------------------------------------------------------------------------------------------
 	if(verbose_)
  		message("FileBinDataSet: allocating targets ...");
  	Sequence **the_targets = (Sequence **)allocator->alloc(sizeof(Sequence *)*io_file->n_sequences);
  	int index = 0;
  	for(int i = 0; i < n_files; i++)
  	{
    		for(int j = 0; j < io_files[i]->n_sequences; j++)
    		{
      			the_targets[index] = new(allocator) Sequence(1, 1);  // Matrix 1x1 ... for the target (real)
      			the_targets[index]->frames[0][0] = targets[i];
      			index++;
    		}
  	}
   

  	// now load the inputs using the init() method and load the targets with setTargets()
  	//------------------------------------------------------------------------------------------- 
 	if(verbose_)
  		message("FileBinDataSet: loading inputs ...");
  	MemoryDataSet::init(io_inputs, io_targets);

 	if(verbose_)
  		message("FileBinDataSet: setting targets ...");
  	setTargets(the_targets, io_file->n_sequences);

 	if(verbose_)
  		message("FileBinDataSet: [%d-%d ] sub-windows of %d examples loaded from %d files with %d targets assigned", offset_window, n_inputs_window, n_examples, n_files, n_files);
}


FileBinDataSet::FileBinDataSet(	char **filenames_class1, int n_files_class1, real target_class1, 
      				char **filenames_class2, int n_files_class2, real target_class2, 
      				int n_inputs_, bool verbose_)
{
	message("FileBinDataSet::FileBinDataSet() (2-class)");
	print("\tClass 1\n");
	print("\t + %d files\n", n_files_class1);
	print("\t + target assigned = %g\n", target_class1);
	print("\tClass 2\n");
	print("\t + %d files\n", n_files_class2);
	print("\t + target assigned = %g\n", target_class2);

  	if(n_files_class1 <= 0)
    		error("FileBinDataSet: check the number of files class1!");

  	if(n_files_class2 <= 0)
    		error("FileBinDataSet: check the number of files class2!");

  	if(n_inputs_ <= 0)
    		error("FileBinDataSet: n_inputs must be positive!");

  	IOSequence *io_inputs = NULL;
  	IOSequence *io_targets = NULL;


	//
  	// Firstly the inputs ...
  	//----------------------------
 	if(verbose_)
  		message("FileBinDataSet: allocating inputs ...");
  	IOBin **io_files = (IOBin **)allocator->alloc(sizeof(IOBin *)*(n_files_class1 + n_files_class2));

  	int n_files_ = 0;
  
  	for(int i = 0; i < n_files_class1 ; i++)
  	{
    		io_files[n_files_] = new(allocator) IOBin(filenames_class1[i], false, -1, false);
    	
 		if(verbose_)
		{
			print("file [%d] = %s\n", i, filenames_class1[i]);
			print("\t+ n_examples      = %d\n", io_files[n_files_]->n_sequences);
			print("\t+ n_inputs        = %d\n", io_files[n_files_]->frame_size);
		}

		n_files_++;
  	}
  
  	for(int i = 0; i < n_files_class2 ; i++)
  	{
    		io_files[n_files_] = new(allocator) IOBin(filenames_class2[i], false, -1, false);
    	
 		if(verbose_)
		{
			print("file [%d] = %s\n", i, filenames_class2[i]);
			print("\t+ n_examples      = %d\n", io_files[n_files_]->n_sequences);
			print("\t+ n_inputs        = %d\n", io_files[n_files_]->frame_size);
		}

		n_files_++;
  	}
  
  	IOMulti *io_file = new(allocator) IOMulti((IOSequence **)io_files, n_files_);

  	io_inputs = io_file;  // the "concatenated" file doesn't contain targets.. (by definition of a bindata file)


	//
  	// ... and then the targets (same real target for all patterns of the same file[i])
  	//-------------------------------------------------------------------------------------------
 	if(verbose_)
  		message("FileBinDataSet: allocating targets ...");
  	Sequence **the_targets = (Sequence **)allocator->alloc(sizeof(Sequence *)*io_file->n_sequences);
  	int index = 0;
  	n_files_ = 0;
  	for(int i = 0; i < n_files_class1; i++)
  	{
    		for(int j = 0; j < io_files[n_files_]->n_sequences; j++)
    		{
      			the_targets[index] = new(allocator) Sequence(1, 1);  // Matrix 1x1 ... for the target (real)
      			the_targets[index]->frames[0][0] = target_class1;
      			index++;
    		}
    		n_files_++;
  	}

  	for(int i = 0; i < n_files_class2; i++)
  	{
    		for(int j = 0; j < io_files[n_files_]->n_sequences; j++)
    		{
      			the_targets[index] = new(allocator) Sequence(1, 1);  // Matrix 1x1 ... for the target (real)
      			the_targets[index]->frames[0][0] = target_class2;
      			index++;
    		}
    		n_files_++;
  	}
   

  	// now load the inputs using the init() method and load the targets with setTargets()
  	//------------------------------------------------------------------------------------------- 
 	if(verbose_)
  		message("FileBinDataSet: loading inputs ...");
  	MemoryDataSet::init(io_inputs, io_targets);

 	if(verbose_)
  		message("FileBinDataSet: setting targets ...");
  	setTargets(the_targets, io_file->n_sequences);

 	if(verbose_)
  		message("FileBinDataSet: %d examples loaded from %d and %d files with %g and %g targets assigned respectively", n_examples, n_files_class1, n_files_class2, target_class1, target_class2);
}


FileBinDataSet::FileBinDataSet(	char **filenames_class, int n_files_class, real target_class, 
      				int n_inputs_, bool verbose_)
{
	message("FileBinDataSet::FileBinDataSet()");
	print("\t+ %d files\n", n_files_class);
	print("\t+ n_inputs expected = %d\n", n_inputs_);
	print("\t+ target assigned = %g\n", target_class);

  	if(n_files_class <= 0)
    		error("FileBinDataSet: check the number of files class1!");

  	if(n_inputs_ <= 0)
    		error("FileBinDataSet: n_inputs must be positive!");

  	IOSequence *io_inputs = NULL;
  	IOSequence *io_targets = NULL;


	//
  	// Firstly the inputs ...
  	//----------------------------
 	if(verbose_)
  		message("FileBinDataSet: allocating inputs ...");
  	IOBin **io_files = (IOBin **)allocator->alloc(sizeof(IOBin *)*n_files_class);

  	int n_files_ = 0;
  
  	for(int i = 0; i < n_files_class ; i++)
  	{
    		io_files[n_files_] = new(allocator) IOBin(filenames_class[i], false, -1, false);
    	
 		if(verbose_)
		{
			print("file [%d] = %s\n", i, filenames_class[i]);
			print("\t+ n_examples      = %d\n", io_files[n_files_]->n_sequences);
			print("\t+ n_inputs        = %d\n", io_files[n_files_]->frame_size);
		}
		
		n_files_++;
  	}
  
  	IOMulti *io_file = new(allocator) IOMulti((IOSequence **)io_files, n_files_);

  	io_inputs = io_file;  // the "concatenated" file doesn't contain targets.. (by definition of a bindata file)

	//
  	// ... and then the targets (same real target for all patterns of the same file[i])
  	//-------------------------------------------------------------------------------------------
 	if(verbose_)
  		message("FileBinDataSet: allocating targets ...");
  	Sequence **the_targets = (Sequence **)allocator->alloc(sizeof(Sequence *)*io_file->n_sequences);
  	int index = 0;
  	n_files_ = 0;
  	for(int i = 0; i < n_files_class; i++)
  	{
    		for(int j = 0; j < io_files[n_files_]->n_sequences; j++)
    		{
      			the_targets[index] = new(allocator) Sequence(1, 1);  // Matrix 1x1 ... for the target (real)
      			the_targets[index]->frames[0][0] = target_class;
      			index++;
    		}
    		n_files_++;
  	}

  	// now load the inputs using the init() method and load the targets with setTargets()
  	//------------------------------------------------------------------------------------------- 
 	if(verbose_)
  		message("FileBinDataSet: loading inputs ...");
  	MemoryDataSet::init(io_inputs, io_targets);

 	if(verbose_)
  		message("FileBinDataSet: setting targets ...");
  	setTargets(the_targets, io_file->n_sequences);

 	if(verbose_)
  		message("FileBinDataSet: %d examples loaded from %d files with %g targets assigned", n_examples, n_files_class, target_class);
}


FileBinDataSet::FileBinDataSet(	char **filenames_class, int n_files_class, real target_class, 
      				int n_inputs_, int offset_window, int n_inputs_window, bool verbose_)
{
	message("FileBinDataSet::FileBinDataSet()");
	print("\t+ %d files\n", n_files_class);
	print("\t+ n_inputs expected = %d\n", n_inputs_);
	print("\t+ offset_window = %d\n", offset_window);
	print("\t+ size_window = %d\n", n_inputs_window);
	print("\t+ target assigned = %g\n", target_class);

  	if(n_files_class <= 0)
    		error("FileBinDataSet: check the number of files class1!");

  	if(n_inputs_ <= 0)
    		error("FileBinDataSet: n_inputs must be positive!");

  	IOSequence *io_inputs = NULL;
  	IOSequence *io_targets = NULL;

	//
  	// Firstly the inputs ...
  	//----------------------------
 	if(verbose_)
  		message("FileBinDataSet: allocating inputs ...");
  	IOSequence **io_files = (IOSequence **)allocator->alloc(sizeof(IOSequence *)*n_files_class);

  	int n_files_ = 0;
  
  	for(int i = 0; i < n_files_class ; i++)
  	{
    		//io_files[n_files_] = new(allocator) IOBin(filenames_class[i], false, -1, false);
	   	IOBin *io_bin = new(allocator) IOBin(filenames_class[i], false, -1, false);
		IOBufferize *io_buffer = new(allocator) IOBufferize(io_bin);
		io_files[i] = new(allocator) IOSub(io_buffer, offset_window, n_inputs_window);
    	
 		if(verbose_)
		{
			print("file [%d] = %s\n", i, filenames_class[i]);
			print("\t+ n_examples      = %d\n", io_files[n_files_]->n_sequences);
			print("\t+ n_inputs        = %d\n", io_files[n_files_]->frame_size);
		}

		n_files_++;
  	}
  
  	IOMulti *io_file = new(allocator) IOMulti(io_files, n_files_);

  	io_inputs = io_file;  // the "concatenated" file doesn't contain targets.. (by definition of a bindata file)

	//
  	// ... and then the targets (same real target for all patterns of the same file[i])
  	//-------------------------------------------------------------------------------------------
 	if(verbose_)
  		message("FileBinDataSet: allocating targets ...");
  	Sequence **the_targets = (Sequence **)allocator->alloc(sizeof(Sequence *)*io_file->n_sequences);
  	int index = 0;
  	n_files_ = 0;
  	for(int i = 0; i < n_files_class; i++)
  	{
    		for(int j = 0; j < io_files[n_files_]->n_sequences; j++)
    		{
      			the_targets[index] = new(allocator) Sequence(1, 1);  // Matrix 1x1 ... for the target (real)
      			the_targets[index]->frames[0][0] = target_class;
      			index++;
    		}
    		n_files_++;
  	}

  	// now load the inputs using the init() method and load the targets with setTargets()
  	//------------------------------------------------------------------------------------------- 
 	if(verbose_)
  		message("FileBinDataSet: loading inputs ...");
  	MemoryDataSet::init(io_inputs, io_targets);

 	if(verbose_)
  		message("FileBinDataSet: setting targets ...");
  	setTargets(the_targets, io_file->n_sequences);

 	if(verbose_)
  		message("FileBinDataSet: [%d-%d ] sub-windows of %d examples loaded from %d files with %g targets assigned", offset_window, n_inputs_window, n_examples, n_files_class, target_class);
}


FileBinDataSet::FileBinDataSet(	char **filenames_class, int n_files_class, 
      				int n_inputs_, int offset_window, int n_inputs_window, bool verbose_)
{
	message("FileBinDataSet::FileBinDataSet()");
	print("\t+ %d files\n", n_files_class);
	print("\t+ n_inputs expected = %d\n", n_inputs_);
	print("\t+ offset_window = %d\n", offset_window);
	print("\t+ size_window = %d\n", n_inputs_window);

  	if(n_files_class <= 0)
    		error("FileBinDataSet: check the number of files class1!");

  	if(n_inputs_ <= 0)
    		error("FileBinDataSet: n_inputs must be positive!");

  	IOSequence *io_inputs = NULL;

	//
  	// Firstly the inputs ...
  	//----------------------------
 	if(verbose_)
  		message("FileBinDataSet: allocating inputs ...");
  	IOSequence **io_files = (IOSequence **)allocator->alloc(sizeof(IOSequence *)*n_files_class);

  	int n_files_ = 0;
  
  	for(int i = 0; i < n_files_class ; i++)
  	{
    		//io_files[n_files_] = new(allocator) IOBin(filenames_class[i], false, -1, false);
	   	IOBin *io_bin = new(allocator) IOBin(filenames_class[i], false, -1, false);
		IOBufferize *io_buffer = new(allocator) IOBufferize(io_bin);
		io_files[i] = new(allocator) IOSub(io_buffer, offset_window, n_inputs_window);
    	
 		if(verbose_)
		{
			print("file [%d] = %s\n", i, filenames_class[i]);
			print("\t+ n_examples      = %d\n", io_files[n_files_]->n_sequences);
			print("\t+ n_inputs        = %d\n", io_files[n_files_]->frame_size);
		}

		n_files_++;
  	}
  
  	IOMulti *io_file = new(allocator) IOMulti(io_files, n_files_);

  	io_inputs = io_file;  // the "concatenated" file doesn't contain targets.. (by definition of a bindata file)

  	// now load the inputs using the init() method
  	//------------------------------------------------------------------------------------------- 
 	if(verbose_)
  		message("FileBinDataSet: loading inputs ...");
  	MemoryDataSet::init(io_inputs, NULL);

 	if(verbose_)
  		message("FileBinDataSet: [%d-%d ] sub-windows of %d examples loaded from %d files with no targets assigned", offset_window, n_inputs_window, n_examples, n_files_class);
}


FileBinDataSet::FileBinDataSet(	char **filenames_class, int n_files_class, int n_inputs_, bool verbose_)
{
	message("FileBinDataSet::FileBinDataSet()");
	print("\t+ %d files\n", n_files_class);
	print("\t+ n_inputs expected = %d\n", n_inputs_);

  	if(n_files_class <= 0)
    		error("FileBinDataSet: check the number of files class1!");

  	if(n_inputs_ <= 0)
    		error("FileBinDataSet: n_inputs must be positive!");

  	IOSequence *io_inputs = NULL;

	//
  	// Firstly the inputs ...
  	//----------------------------
 	if(verbose_)
  		message("FileBinDataSet: allocating inputs ...");
  	IOSequence **io_files = (IOSequence **)allocator->alloc(sizeof(IOSequence *)*n_files_class);

  	int n_files_ = 0;
  
  	for(int i = 0; i < n_files_class ; i++)
  	{
    		io_files[n_files_] = new(allocator) IOBin(filenames_class[i], false, -1, false);
    	
 		if(verbose_)
		{
			print("file [%d] = %s\n", i, filenames_class[i]);
			print("\t+ n_examples      = %d\n", io_files[n_files_]->n_sequences);
			print("\t+ n_inputs        = %d\n", io_files[n_files_]->frame_size);
		}

		n_files_++;
  	}
  
  	IOMulti *io_file = new(allocator) IOMulti(io_files, n_files_);

  	io_inputs = io_file;  // the "concatenated" file doesn't contain targets.. (by definition of a bindata file)

  	// now load the inputs using the init() method
  	//------------------------------------------------------------------------------------------- 
 	if(verbose_)
  		message("FileBinDataSet: loading inputs ...");
  	MemoryDataSet::init(io_inputs, NULL);

 	if(verbose_)
  		message("FileBinDataSet: %d examples loaded from %d files with no targets assigned", n_examples, n_files_class);
}


void FileBinDataSet::info(bool display_)
{
	message("FileBinDataSet::info():");
	print("\t+ n_examples = %d\n", n_examples);
	print("\t+ n_inputs = %d\n", n_inputs);
	print("\t+ n_targets = %d\n", n_targets);
      			
	print("\n");

  	if(display_)
  	{
    		for(int i=0; i<n_examples; i++)
    		{
      			// Targets
      			print("[");
      			for(int j=0; j<n_targets; j++)
				print(" %g ", targets_array[i]->frames[0][j]);
      			print("] ");
      
			// Inputs
      			for(int j=0; j<n_inputs; j++)
				print("%g|", inputs_array[i]->frames[0][j]);
      			print("\n\n");
    		}
  	}
}


FileBinDataSet::~FileBinDataSet()
{
}

}
