#include "Random.h"

#include "ImageGray.h"

#include "ipFaceExtract.h"
#include "ipRotate.h"
#include "ipScale_yx.h"

#include "trigonometry.h"

namespace Torch {


/* compute the new bbx according to shift, scale.  */
real computeRandomBox(int shift, int scale, int rotat, int *bbx_x, int *bbx_y, int *bbx_w, int *bbx_h, int fm_w, int fm_h, int im_width_, int im_height_)
{
   // ====== total number of possible different subimages (shift, scale, rotation)  ======
   int max_random = 1;
   if(shift) max_random *= 9 * shift;
   if(scale) max_random *= 3 * scale;
   if(rotat) max_random *= 3 * rotat;
	
   int random_value = (int)(max_random * (Random::uniform()));  // [0;max_random[

   //=====  compute scale_dir, scale_val, shift_dir, shift_val  =====
   int r_val = random_value;
   int max = max_random;
   int rotat_dir = 0, rotat_val = 0, scale_dir = 0, scale_val = 0, shift_dir = 0, shift_val = 0;


   // rotat_dir = 0: -angle; rotat_dir = 1: +angle;
   // rotat_val = 1: 5 degrees of rotation
   if(rotat)
   {
	rotat_val = r_val / (max/rotat);
	max = max / rotat;
	r_val = r_val - (rotat_val * max);
	
	rotat_dir = r_val / (max/3);  // 2 rotation directions + no dir
	max = max / 3;
	r_val = r_val - (rotat_dir * max);
   }
   
   // scale_dir = 0: zoom in; scale_dir = 1: zoom out;
   // scale_val = 1: zoom +/- 10% of the width of the bbx
   if(scale) 
   {
	   scale_val = r_val / (max/scale);
	   max = max / scale;
	   r_val = r_val - (scale_val * max);

	   scale_dir = r_val / (max/3);  // 2 scale directions + no dir
           max = max / 3;
           r_val = r_val - (scale_dir * max);
   }

   // shift_dir: 8 directions (see code)
   // shift_val = 1: shift of 1 pixel in the cropped downsized image.
   if(shift) 
   {
	   shift_val = r_val / (max/shift);
	   max = max / shift;
	   r_val = r_val - (shift_val * max);

	   shift_dir = r_val / (max/9);  // 8 shift directions + no dir
	   max = max / 9;
   }
	
   if(max != 1) error("FaceExtract::computeRandomBox  max != 1 (%d)", max);
   
   //=====   update the bbx according to the perturbations  =====
   real angle = 0.0;
   if(rotat)
   {
	switch(rotat_dir)
	{
	   case 0: angle = -((rotat_val+1)*5); break;
	   case 1: angle = 0.0;                break;
	   case 2: angle = (rotat_val+1)*5;    break;
	   default: error("Incorrect value for rotat_dir.");
	}
   }
   
   real scale_factor = 1.0;
   if(scale)
   {
	switch(scale_dir)
	{
	   case 0: scale_factor = 1 - ((scale_val+1)*0.1); break; 
	   case 1: scale_factor = 1;                       break;
	   case 2: scale_factor = 1 + ((scale_val+1)*0.1); break;
	   default: error("Incorrect value for scale_dir.");  
	}
   }
   
   int dx = 0, dy = 0;	
   if(shift) 
   {
	int shift_x = (int)( (real)(*bbx_w) / fm_w * (shift_val+1) );
	int shift_y = (int)( (real)(*bbx_h) / fm_h * (shift_val+1) );
	
	switch(shift_dir)
	{
	   case 0: dx = 0;        dy = 0;         break;
	   case 1: dx = -shift_x; dy = -shift_y;  break;
	   case 2: dx = 0;        dy = -shift_y;  break;
	   case 3: dx = shift_x;  dy = -shift_y;  break;
	   case 4: dx = shift_x;  dy = 0;         break;
	   case 5: dx = shift_x;  dy = shift_y;   break;
	   case 6: dx = 0;        dy = shift_y;   break;
	   case 7: dx = -shift_x; dy = shift_y;   break;
	   case 8: dx = -shift_x; dy = 0;         break;
	   default: error("Incorrect value for shift_dir.");
	}
   }

   // current bbx values
   int x = (*bbx_x);
   int y = (*bbx_y);
   int width  = (*bbx_w);
   int height = (*bbx_h);

   int new_x = x, new_y = y, new_width = width, new_height = height;
   
   // shift: update bbx values
   if(shift) {
	new_x = x + dx;
	new_y = y + dy;
   }

   // scale: update bbx values
   if(scale) {
	new_width  = FixI(width * scale_factor);
	new_height = FixI(height * scale_factor);
	new_x = new_x - FixI((new_width - width)/2);
	new_y = new_y - FixI((new_height - height)/2);
   }

   *bbx_x = new_x;
   *bbx_y = new_y;
   *bbx_w = new_width;
   *bbx_h = new_height;


   return angle;
}

	
ipFaceExtract::ipFaceExtract(int width_in_, int height_in_, int width_out_, int height_out_, const char *coding_, FaceModel *face_model_, real angle_offset_, int shift_, int scale_, int rotat_, int n_selected_) : ipCore(coding_)
{
	width_in = width_in_;
	height_in = height_in_;
	width_out = width_out_;
	height_out = height_out_;
	face_model = face_model_;
	angle_offset = angle_offset_;

	shift = shift_;
	scale = scale_;
	rotat = rotat_;
	n_selected = n_selected_;

	if(strcmp(coding,"gray") != 0)
		error("ipFaceExtract supports only 'gray' coding. (because of ipScale_yx");
		
	seq_out = new(allocator) Sequence(n_selected, width_out*height_out*n_planes);

	for(int j = 0 ; j < n_selected ; j++)
		for(int i = 0 ; i < seq_out->frame_size ; i++)
			seq_out->frames[j][i] = 0.0;
}
	
int ipFaceExtract::getWidthOut(void)
{
	return width_out;
}

int ipFaceExtract::getHeightOut(void)
{
	return height_out;
}

void ipFaceExtract::process(Sequence *data_)
{
	// ====== horizontal angle between the two eyes  ======
	double angle = face_model->getAngle();
	sPoint2D p = face_model->getCenter();
	double deg_angle = radian2degree(angle);
	//message("Horizontal angle (degres: %.1f)", angle * 180 / M_PI);
	//message("Horizontal angle (degres: %g)", deg_angle);
	
	// ====== rotate the image (needs angle in degrees!) ======
	//ipRotate *rot_machine = new(allocator) ipRotate(angle * 180 / M_PI, width_in, height_in, "gray");
	ipRotate *rot_machine = new(allocator) ipRotate(deg_angle, width_in, height_in, "gray");
	rot_machine->setBOption("verbose", verbose);

	if(p.x >= 0.0 && p.y >= 0.0)
		rot_machine->setCenter((int) p.x, (int) p.y);
	else rot_machine->setCenter(FixI(width_in/2.0), FixI(height_in/2.0));
	rot_machine->process(data_);

	width_in = rot_machine->getWidthOut();
	height_in = rot_machine->getHeightOut();

	Sequence *data_rot_ = new(allocator) Sequence(&rot_machine->seq_out->frames[0], 1, rot_machine->seq_out->frame_size);

	if(angle != 0.0)
		// ====== update the groundtruth coordinates ======
		face_model->rotate(rot_machine);

	// ======  compute the bbx according to the face model  ======
	if(!face_model->ldm2bbx())
		error("ipFaceExtract::process() impossible to create the face model from eyes coordinates.");

	// ======   ======
	// if rotation needed, we have another ipRotate...
	ipRotate *rot_machine_tmp = NULL;
	
	for(int i=0; i<n_selected; i++)
	{
	   int bbx_x = (int)face_model->bbx_points[0].x;
	   int bbx_y = (int)face_model->bbx_points[0].y;
	   int bbx_w = face_model->bbx_width;
	   int bbx_h = face_model->bbx_height;
	   int bbx_cx = (int)face_model->bbx_center.x;
	   int bbx_cy = (int)face_model->bbx_center.y;

	   // compute new bbx according to shift and scale.
	   real angle = computeRandomBox(shift, scale, rotat, &bbx_x, &bbx_y, &bbx_w, &bbx_h, getWidthOut(), getHeightOut(), width_in, height_in);
	
	   // temp buffer for the cropped image
	   Sequence *buffer_ = new(allocator) Sequence(1, bbx_w * bbx_h);

	   angle += angle_offset;
	   
	   if(angle)
	   {
		rot_machine_tmp = new(allocator) ipRotate(angle, width_in, height_in, "gray");
	        rot_machine_tmp->setBOption("verbose", verbose);
		rot_machine_tmp->setCenter(bbx_cx, bbx_cy);
		rot_machine_tmp->process(data_rot_);

		int bbx_cx_, bbx_cy_;
		rot_machine_tmp->rotate(bbx_cx, bbx_cy, &bbx_cx_, &bbx_cy_);
		
		cropGray(rot_machine_tmp->seq_out->frames[0], rot_machine_tmp->getWidthOut(), rot_machine_tmp->getHeightOut(), bbx_cx_ - bbx_w/2, bbx_cy_ - bbx_h/2, bbx_w, bbx_h, buffer_->frames[0]);		
	
		allocator->free(rot_machine_tmp);
	   } 
	   else
	   	cropGray(data_rot_->frames[0], width_in, height_in, bbx_x, bbx_y, bbx_w, bbx_h, buffer_->frames[0]);

	   // subsample the cropped image
	   ipScale_yx *scale_machine = new(allocator) ipScale_yx(bbx_w, bbx_h, "gray", getWidthOut(), getHeightOut());
	   scale_machine->setBOption("verbose", verbose);
	   scale_machine->process(buffer_);

	   // copy the cropped subsampled image into output
	   real *ptr_tmp_ = scale_machine->seq_out->frames[0];
	   for(int j=0; j< seq_out->frame_size; j++) seq_out->frames[i][j] = FixI(ptr_tmp_[j]); 
	   
	   allocator->free(scale_machine);
	   allocator->free(buffer_);
	}

	allocator->free(data_rot_);
	allocator->free(rot_machine);
}

ipFaceExtract::~ipFaceExtract()
{
}

}

